﻿using UnityEngine;
using System.Collections;

public class GameManager : MonoBehaviour {

	private static GameManager instance = null;

	public Color[] colorSet;
	public float switchTimer = 4.0f;
	public GameObject backPlane;

	public float musicDelay=0.5f;
	private AudioSource music;

	private int[] score;	
	private int currColor=0;
	private int nextColor=0;
	private float currTimer;
	private GameObject oldBackPlane;
	private GameObject newBackPlane;
	private AsyncOperation async;

	// Use this for initialization
	void Awake () {
		if (instance != null && instance != this) {
			Destroy(this.gameObject);
			return;
		} else {
			instance = this;
		}
		DontDestroyOnLoad(this.gameObject);
	}

	// Use this for initialization
	void Start () {
		StartLoading();
		score = new int[]{0,0,0,0,0,0,0,0};
		currTimer = 0.0f;
		newBackPlane = (GameObject)Instantiate(backPlane,new Vector3(transform.position.x,transform.position.y,10f),backPlane.transform.rotation);
		currColor = randColor();
		nextColor = currColor;
		newBackPlane.renderer.material.color=colorSet[currColor];
		newBackPlane.transform.parent = transform;
		music = (AudioSource)this.GetComponent(typeof(AudioSource));
		music.PlayDelayed(musicDelay);
	}

	// Update is called once per frame
	void FixedUpdate () {
		//Debug.Log (Time.deltaTime);
		if (currTimer >= switchTimer){
			colorSwitch();
			currTimer = 0f;
		}else{
			currTimer+=Time.deltaTime;
		}
	}

	private void StartLoading(){
		int nextLevel = Random.Range(1, Application.levelCount - 1);
		while (nextLevel == Application.loadedLevel){
			nextLevel = Random.Range(1, Application.levelCount - 1);
		}
		async = Application.LoadLevelAsync(nextLevel);
		async.allowSceneActivation = false;
	}

	public void LoadNextLevel(){
		async.allowSceneActivation = true;
		//int nextLevel = Random.Range(0, Application.levelCount - 1);
		//Application.LoadLevel(nextLevel);
		StartLoading ();
	}

	//Switch current color
	private void colorSwitch(){
		nextColor = randColor ();
		newBackPlane = (GameObject)Instantiate(backPlane,new Vector3(transform.position.x,transform.position.y,8f),backPlane.transform.rotation);
		newBackPlane.renderer.material.color=colorSet[nextColor];
		newBackPlane.transform.parent = transform;
	}

	//Generate random color
	private int randColor(){
		int tempColor = currColor;
		while (tempColor == currColor){
			tempColor = Random.Range(0,colorSet.Length);
		}
		return tempColor;
	}

	//Get current color
	public int getCurrColor(){
		return currColor;
	}

	//get random color
	public int getRandColor(){
		return Random.Range(0,colorSet.Length);
	}

	public Color getColor(int colorCode){
		return colorSet[colorCode];
	}

	//Remove old background
	public void removeOldBack(){
		Destroy(oldBackPlane);
		currColor = nextColor;
		oldBackPlane = newBackPlane;
	}

	public void updateScore(int playerNum){
		score[playerNum]+=1;
	}

	//Check if there are no remaining players
	public void checkRemainingPlayers(){
		GameObject[] playerList;
		playerList=GameObject.FindGameObjectsWithTag("Player");
		if (playerList.Length <= 1){
			LoadNextLevel();
		}
	}
	public int getScore(int playerNum){
		return score[playerNum];
	}

}
