﻿using UnityEngine;
using System.Collections;

public class MovementController : MonoBehaviour {

	public float jumpStrength = 5.0f;	//Jump Strength
	public float walkSpeed = 1.0f;		//Walk accelleration
	public float maxWalk = 1.0f;		//Maximum walk speed
	public float maxFall = 1.0f;
	public float boostSpeed = 150.0f; 			//This is a boost speed
	public float boostTime = 5f;				//Boost Time

	public GameObject punchObj;

	public int playerNum=0;
	public Color playerColor;

	public SpriteRenderer sprite;

	public AudioClip jump1Clip;
	public AudioClip jump2Clip;
	public AudioClip jump3Clip;
	public AudioClip stunClip;
	public AudioClip punchClip;

	private float stunTimer = 0.0f;
	private float punchTimer = 0.0f;
	private float invulnTimer = 0.0f;
	private bool isStunned = false;

	private int score;
	private float scoreTimer = 4.0f;
	private GUIStyle guiStyle;

	private bool hasBoost;
	private bool isGrounded;
	private bool isBoosting;
	private bool lookingRight=true;

	private Animator animator;

	// Use this for initialization
	void Start () {
		isGrounded = true;
		hasBoost = true;
		animator=(Animator)sprite.GetComponent(typeof(Animator));

		score = ((GameManager)GameObject.Find("_GameManager").GetComponent(typeof(GameManager))).getScore(playerNum);

		guiStyle = new GUIStyle();
		guiStyle.fontStyle=FontStyle.Bold;
		guiStyle.normal.textColor = playerColor;
	}
	
	// FixedUpdate is called once per fixed frame
	void FixedUpdate () {
		checkGrounded();
		if (isBoosting){
			boostTime -= Time.fixedDeltaTime;
			if (boostTime <= 0f){
				isBoosting=false;
				//Debug.Log("Not Boosting.");
				if (rigidbody.velocity.x < 0){
					rigidbody.velocity = new Vector3(-maxWalk,rigidbody.velocity.y,rigidbody.velocity.z);
				}else{
					rigidbody.velocity = new Vector3(maxWalk,rigidbody.velocity.y,rigidbody.velocity.z);
				}

			}
		}
		if (rigidbody.velocity.y < -maxFall){rigidbody.velocity = new Vector3(rigidbody.velocity.x,-maxFall,rigidbody.velocity.z);}
		if (isStunned){
			stunTimer -= Time.deltaTime;
			if (stunTimer <= 0.0f){
				isStunned = false;
			}
		}
		if (punchTimer > 0.0f){
			punchTimer -= Time.deltaTime;
			if (punchTimer <= 0.0f){
				animator.SetBool ("isPunching", false);
			}
		}
		if (invulnTimer > 0.0f){
			invulnTimer -= Time.deltaTime;
		}
		if (scoreTimer > 0.0f){
			scoreTimer -= Time.deltaTime;
		}
		if (lookingRight){
			transform.localScale = new Vector3(1.0f,1.0f,1.0f);
		}else{
			transform.localScale = new Vector3(-1.0f,1.0f,1.0f);
		}
		updateState();
		checkDead();
	}

	void OnGUI (){
		if (scoreTimer>0.0f){
			Vector3 screenPos = Camera.main.WorldToScreenPoint(transform.position);
			GUI.Label(new Rect(screenPos.x, Screen.height-screenPos.y-50.0f, 100f, 20f), (""+score),guiStyle);
		}
	}

	public void move(Vector3 moveDir){
		if (!isStunned){
			bool tempCheck;
			if (moveDir.x>=0){tempCheck = checkWall(1);}
			else{tempCheck = checkWall(-1);}
			if (tempCheck){
				rigidbody.velocity += moveDir*walkSpeed;
				if (rigidbody.velocity.x > 0){lookingRight = true;}
				if (rigidbody.velocity.x < 0){lookingRight = false;}
				if (Mathf.Abs(rigidbody.velocity.x) > maxWalk){
					if (!isBoosting){
						float tempY = rigidbody.velocity.y;
						rigidbody.velocity = rigidbody.velocity.normalized * maxWalk;
						rigidbody.velocity = new Vector3(rigidbody.velocity.x,tempY,rigidbody.velocity.z);
					}
				}
			}
		}
	}

	private bool checkWall(int dir){
		bool tempCheck = true;
		RaycastHit hit = new RaycastHit();
		if (dir >= 0){
			if (Physics.SphereCast(transform.position,0.4f,transform.right,out hit,1.0f)){
				if (hit.transform.gameObject.tag != "Player"){tempCheck = false;}
			}
		}else{
			if (Physics.SphereCast(transform.position,0.4f,-transform.right,out hit,1.0f)){
				if (hit.transform.gameObject.tag != "Player"){tempCheck = false;}
			}
		}
		return tempCheck;
	}

	public void stun(float stunTime){
		if (invulnTimer <= 0.0f){
			AudioSource.PlayClipAtPoint(stunClip,transform.position);
			isStunned = true;
			stunTimer = stunTime;
			invulnTimer = stunTime +1.0f;
		}
	}

	//Make the player punch
	public void punch(){
		if (!isStunned){
			Punch tempPunch;
			animator.SetBool ("isPunching", true);
			punchTimer = 0.1f;
			AudioSource.PlayClipAtPoint(punchClip,transform.position);
			if (lookingRight){
				tempPunch = (Punch)((GameObject)Instantiate(punchObj,new Vector3(transform.position.x+0.25f,transform.position.y,transform.position.z),Quaternion.identity)).GetComponent(typeof(Punch));
			}else{
				tempPunch = (Punch)((GameObject)Instantiate(punchObj,new Vector3(transform.position.x-0.25f,transform.position.y,transform.position.z),Quaternion.identity)).GetComponent(typeof(Punch));
			}
			tempPunch.setColor(playerNum);
		}
	}

	//Make the player jump
	public void jump(){
		if (!isStunned){
			if (isGrounded){
				rigidbody.velocity = new Vector3(rigidbody.velocity.x,jumpStrength,rigidbody.velocity.z);
				int tempRand = Random.Range(0,3);
				if (tempRand == 0){
					AudioSource.PlayClipAtPoint(jump1Clip,transform.position,0.2f);
				}else if (tempRand ==1){
					AudioSource.PlayClipAtPoint(jump2Clip,transform.position,0.2f);
				}else{
					AudioSource.PlayClipAtPoint(jump3Clip,transform.position,0.2f);
				}
			}
		}
	}

	//Make the player boost
	public void boost(){
		if (!isStunned){
			if (hasBoost == true){
				Debug.Log("Boosting.");
				isBoosting = true;
				hasBoost = false;
				rigidbody.velocity = new Vector3(0,rigidbody.velocity.y,rigidbody.velocity.z);
			}
		}
	}



	//Is the player on the ground?
	private void checkGrounded(){
		//Old system with raycast.  Less accurate
		/*RaycastHit hit = new RaycastHit();
		if (Physics.Raycast(transform.position,-transform.up,out hit,1)){
			isGrounded = true;
		}else{
			isGrounded = false;
		}*/
		RaycastHit hit = new RaycastHit();
		if (Physics.SphereCast(transform.position,0.4f,-transform.up,out hit,1.0f)){
			isGrounded = true;
		}else{
			isGrounded = false;
		}
	}

	public void setColor(Color newColor){
		playerColor = newColor;
		sprite.color=playerColor;
	}

	private void updateState(){
		animator.SetFloat ("speed", Mathf.Abs(rigidbody.velocity.x));
		animator.SetBool ("isGrounded", isGrounded);
		animator.SetBool ("isStunned", isStunned);
	}

	private void checkDead(){
		if (((Camera.main.transform.position.x-transform.position.x) >= 25f)||((Camera.main.transform.position.y-transform.position.y) >= 15f)){
			Debug.Log ("Player #" + (playerNum+1) + " has fallen behind.");
			((GameManager)GameObject.Find("_GameManager").GetComponent(typeof(GameManager))).checkRemainingPlayers();
			Destroy (this.gameObject);
		}
	}
}
